// Mesure de temperature avec SHT21
// Fichier SHT21.cpp
#include <iostream> 
#include <bcm2835.h>
#include <math.h>
using namespace std;

#define clk_div BCM2835_I2C_CLOCK_DIVIDER_2500

// g++ SHT21.cpp -l bcm2835 -o SHT21

void init_I2c_bcm2835()
{
	if (!bcm2835_init())
    		{
      		printf("bcm2835_init failed. Are you running as root??\n");
    		}
      	if (!bcm2835_i2c_begin())
      		{
        	printf("bcm2835_i2c_begin failed. Are you running as root??\n");
      		}
	bcm2835_i2c_setClockDivider(clk_div);			// Division de l'horloge Rpi pour obtenir une vitesse de 100KHz
}

int main(void)
{
	float mesureT, mesureRH;
	float alpha, Tr;
	int ST, SRH;
	unsigned int slave_address=0x40; // Adresse du SHT21
	char i2cOut[3]; 	// Tableau des données I2C à sortir 
	char i2cIn[2];		// Tableau des données I2C entrées

	init_I2c_bcm2835();
	bcm2835_i2c_setSlaveAddress(slave_address);	// Affectation de l'adresse de l'esclave 
	cout<<"Mesures sur le SHT20/SHT21 : "<<endl;


	i2cOut[0] = 0xFE;				// Soft reset
	bcm2835_i2c_write(i2cOut, 1); 	// 

	delay(1000);					// 1 seconde avant première lecture

	while(1)
		{
		// Mesure température
		i2cOut[0] = 0xF3; 				// Octet  lecture température (no hold master) 
		bcm2835_i2c_write(i2cOut, 1); 	// Commande lecture température
		delay(100);						// 100 msec de délais en dessous cela dysfonctionne
		bcm2835_i2c_read(i2cIn,2);	// Lecture des 2 octets du resultat
		//cout<<"i2cIn[0] = "<< (int)i2cIn[0]<< "      i2cIn[1] = "<< (int)i2cIn[1]<<endl;
		ST=256*(int)i2cIn[0] + (int)(i2cIn[1]&0b11111100);	// Signal Temperature
		//cout<<"ST = "<< ST<<endl;
		mesureT= (-46.85 +175.72 * ST/65536);
		cout<<"Temperature du SHT20/21 : "<<mesureT<<" deg."<<endl;

		// Mesure humidité relative
		i2cOut[0] = 0xF5; 				// Octet  lecture humidité relative (no hold master) 
		bcm2835_i2c_write(i2cOut, 1); 	// Commande lecture humidité relative
		delay(100);						// 100 msec de délais en dessous cela dysfonctionne
		bcm2835_i2c_read(i2cIn,2);	// Lecture des 2 octets du resultat
		//cout<<"i2cIn[0] = "<< (int)i2cIn[0]<< "      i2cIn[1] = "<< (int)i2cIn[1]<<endl;
		SRH=256*(int)i2cIn[0] + (int)(i2cIn[1]&0b11110000);	// Signal humidité relative
		//cout<<"SRH= "<< SRH<<endl;
		mesureRH= (-6.00 +125.00 * SRH/65536);
		cout<<"Humidité relative du SHT20/21 : "<<mesureRH<<" %"<<endl;

		// Calcul du point de rosée
		alpha=(17.27*mesureT)/(237.7+mesureT)+log(mesureRH/100);
		Tr = (237.7*alpha)/(17.27-alpha);
		cout<<"Point de rosée : "<<Tr<<" deg."<<endl;	
	
		cout<<endl;
		delay(1000);									// 1 seconde entre chaque conversion
   		}

	bcm2835_close();
	return 0;
}
